/*
 * Copyright The University of Chicago
 * All Rights Reserved
 */

#ifndef GLOBUS_XIO_UNIX_H
#define GLOBUS_XIO_UNIX_H

/**
 * @file globus_xio_unix.h
 * @brief Header file for XIO Unix Domain Socket Driver
 * This Globus XIO Unix Domain Socket Driver is a transport-level driver
 * that provides an interface to unix domain sockets that are bound to the 
 * file system.
 *
 * This driver includes supports the following socket options, implemented
 * as attributes:
 * - listener backlog
 * - send buffer size
 * - receive buffer size
 *
 * Additionally, it supports sending and receiving file descriptors
 * between unrelated processes via the socket. This is implemented using
 * the data descriptor interface.
 */
#include "globus_xio_system.h"

#ifdef __cplusplus
extern "C" {
#endif


/** doxygen varargs filter stuff
 * GlobusVarArgDefine(
 *      attr, globus_result_t, globus_xio_attr_cntl, attr, driver)
 * GlobusVarArgDefine(
 *      handle, globus_result_t, globus_xio_handle_cntl, handle, driver)
 * GlobusVarArgDefine(
 *      server, globus_result_t, globus_xio_server_cntl, server, driver)
 * GlobusVarArgDefine(
 *      dd, globus_result_t, globus_xio_data_descriptor_cntl, dd, driver)
 */

/**
 * Unix Domain Socket driver specific cntls
 * @ingroup globus_xio_unix_domain_driver_cntls
 */
typedef enum
{
    /** GlobusVarArgEnum(attr)
     * Set the unix domain socket path to bind to.
     * @ingroup globus_xio_unix_domain_driver_cntls
     * Used only on attrs for @ref globus_xio_server_create().
     * 
     * @param path_name
     *      The  name to use to create the listener. If an object
     *      exists in that path, it will be removed before the socket is
     *      bound to the path.
     *
     * string opt: path=<em>string</em>
     */
    /* const char *                     path_name */
    GLOBUS_XIO_UNIX_SET_PATH,
    
    /** GlobusVarArgEnum(attr)
     * Get the path to bind to.
     * @ingroup globus_xio_unix_domain_driver_cntls
     * 
     * @param service_name_out
     *      A pointer to the path name will be stored here  If none is set,
     *      NULL will be passed back.  Otherwise, the name will be
     *      duplicated with strdup() and the user should call free() on it.
     */
    /* char **                          path_name_out */
    GLOBUS_XIO_UNIX_GET_PATH,
    
    /** GlobusVarArgEnum(attr)
     * Set the listener backlog on a server.
     * @ingroup globus_xio_unix_domain_driver_cntls
     * Used only on attrs for @ref globus_xio_server_create(). The default
     * backlog is -1 (system maximum)
     * 
     * @param listener_backlog
     *      This indicates the maximum length of the system's queue of 
     *      pending connections. Any connection attempts when the queue is
     *      full will fail. If backlog is equal to -1, then the system-specific
     *      maximum queue length will be used.
     * string opt: backlog=<em>int</em>
     */
    /* int                              listener_backlog */
    GLOBUS_XIO_UNIX_SET_BACKLOG,
    
    /** GlobusVarArgEnum(attr)
     * Get the listener backlog on an attr.
     * @ingroup globus_xio_unix_domain_driver_cntls
     * 
     * @param listener_backlog_out
     *      The backlog will be stored here.
     */
    /* int *                            listener_backlog_out */
    GLOBUS_XIO_UNIX_GET_BACKLOG,
    
    /** GlobusVarArgEnum(attr, handle, server)
     * Get the socket handle on an attr, handle, or server.
     * @ingroup globus_xio_unix_domain_driver_cntls
     * 
     * @param handle_out
     *      The socket will be stored here. If none is set,
     *      GLOBUS_XIO_UNIX_INVALID_HANDLE will be set.
     */
    /* globus_xio_system_socket_t *     handle_out */
    GLOBUS_XIO_UNIX_GET_HANDLE,
    
    /** GlobusVarArgEnum(attr)
     * Set the socket to use for a handle or server.
     * @ingroup globus_xio_unix_domain_driver_cntls
     * Used only on attrs for @ref globus_xio_server_create() or
     * @ref globus_xio_register_open().
     * 
     * @param handle
     *      Use this handle (fd or SOCKET) for the listener or connection.
     *      Note:  close() will not be called on this handle.
     * string opt: fd=<em>int</em>
     */
    /* globus_xio_system_socket_t        handle */
    GLOBUS_XIO_UNIX_SET_HANDLE,
    
    /** GlobusVarArgEnum(attr, handle)
     * Set the socket send buffer size.
     * @ingroup globus_xio_unix_domain_driver_cntls
     * Used on attrs for @ref globus_xio_server_create(), 
     * @ref globus_xio_register_open() and with @ref globus_xio_handle_cntl()
     * to set the size of the send buffer used on the socket.
     * 
     * @param sndbuf
     *      The send buffer size in bytes to use. (default is system specific)
     *
     * string opt: sndbuf=<em>formatted int</em>
     */
    /* int                              sndbuf */
    GLOBUS_XIO_UNIX_SET_SNDBUF,
    
    /** GlobusVarArgEnum(attr, handle)
     * Get the socket send buffer size on the attr or handle.
     * @ingroup globus_xio_unix_domain_driver_cntls
     * 
     * @param sndbuf_out
     *      The send buffer size will be stored here.
     */
    /* int *                            sndbuf_out */
    GLOBUS_XIO_UNIX_GET_SNDBUF,
    
    /** GlobusVarArgEnum(attr, handle)
     * Set the socket socket receive buffer size.
     * @ingroup globus_xio_unix_domain_driver_cntls
     * Used on attrs for @ref globus_xio_server_create(), 
     * @ref globus_xio_register_open() and with @ref globus_xio_handle_cntl()
     * to set the size of the receive buffer used on the socket.
     * 
     * @param rcvbuf
     *      The receive buffer size in bytes. (default is system specific)
     *
     * string opt: rcvbuf=<em>formatted int</em>
     */
    /* int                              rcvbuf */
    GLOBUS_XIO_UNIX_SET_RCVBUF,
    
    /** GlobusVarArgEnum(attr, handle)
     * Get the socket receive buffer size on the attr or handle.
     * @ingroup globus_xio_unix_domain_driver_cntls
     * 
     * @param rcvbuf_out
     *      The receive buffer size will be stored here.
     */
    /* int *                            rcvbuf_out */
    GLOBUS_XIO_UNIX_GET_RCVBUF,
    
    /**GlobusVarArgEnum(dd)
     * Send file descriptors as ancillary data.
     * @ingroup globus_xio_unix_driver_cntls
     * Used only for data descriptors to write calls.
     *
     * @param descriptors
     *      An array of file descriptors.
     * @param descriptors_count
     *      Length of the descriptors array.
     */
    /* int                             *descriptors,
       size_t                           descriptors_count */
    GLOBUS_XIO_UNIX_SEND_DESCRIPTORS,

    /**GlobusVarArgEnum(dd)
     * Receive file descriptors as ancillary data.
     * @ingroup globus_xio_unix_driver_cntls
     * Used only for data descriptors to read calls.
     *
     * @param descriptors
     *      Pointer to an array to hold the received file descriptors
     * @param descriptors_count
     *      Pointer to the length of the descriptors array.
     */
    /* int                            **descriptors,
       size_t                          *descriptors_count */
    GLOBUS_XIO_UNIX_RECV_DESCRIPTORS,
    
} globus_xio_unix_cmd_t;

#ifdef __cplusplus
}
#endif

#endif /* GLOBUS_XIO_UNIX_H */
